const mongoose = require('mongoose');
const express = require('express');
const cookieParser = require('cookie-parser');
const fs = require('fs');
const crypto = require('crypto');

const connection_string = 'mongodb://127.0.0.1/responsibilities';

mongoose.connect(connection_string);
mongoose.connection.on('error', () => {
  console.log('There was a problem connecting to mongoDB');
});

/**
 * Specify the schema for the database, including items and categories.
 */
 var UserSchema = new mongoose.Schema({
  username: String,
  password: String,
  faveColor: String
});
var User = mongoose.model('User', UserSchema );
var ItemSchema = new mongoose.Schema( {
    name: String,
    difficulty: Number,
    stat: String,
    username: String,
    category: String
});
var Item = mongoose.model('Item', ItemSchema);
var CategorySchema = new mongoose.Schema( {
  name: String,
  priority: Number,
  color: String,
  username: String
});
var Category = mongoose.model('Category', CategorySchema);

let sessions = {};

function addSession(username) {
  let sid = Math.floor(Math.random() * 1000000000);
  let now = Date.now();
  sessions[username] = {id: sid, time: now};
  return sid;
}

function removeSessions() {
  let now = Date.now();
  let usernames = Object.keys(sessions);
  for (let i = 0; i < usernames.length; i++) {
    let last = sessions[usernames[i]].time;
    //if (last + 120000 < now) {
    if (last + 20000 < now) {
      delete sessions[usernames[i]];
    }
  }
  console.log(sessions);
}

setInterval(removeSessions, 2000);

/**
 * Initialize the express app and configure with various features 
 * such as JSON parsing, static file serving, etc.
 */
const app = express();
app.use(cookieParser());    
//app.get('/', (req, res) => { res.redirect('/app/index.html'); });
app.use(express.json())
//app.use(parser.text({type: '*/*'}));

function authenticate(req, res, next) {
  let c = req.cookies;
  console.log('auth request:');
  console.log(req.cookies);
  if (c != undefined) {
    if (sessions[c.login.username] != undefined && 
      sessions[c.login.username].id == c.login.sessionID) {
      next();
    } else {
      res.redirect('/account/index.html');
    }
  }  else {
    res.redirect('/account/index.html');
  }
}

app.use('/app/*', authenticate);
app.get('/app/*', (req, res, next) => { 
  console.log('another');
  next();
});
app.use(express.static('public_html'))

app.get('/A', (req, res, next) => {
  console.log('A');
  next();
});

app.get('/A', (req, res, next) => {
  console.log('B');
  next();
});

app.get('/A', (req, res, next) => {
  console.log('C');
  res.end('Made it to C!');
});

/**
 * Get all of the items that correspond with a particular catagory.
 * The category is specified as a part of the URL after /items/.
 */
app.get('/items/:cat', (req, res, next) => {
    let cat = req.params.cat;
    let p1 = Item.find({category: cat}).exec();
    p1.then( (results) => { 
      res.end( JSON.stringify(results) );
    });
    p1.catch( (error) => {
      console.log(error);
      res.end('FAIL');
    });
});

/**
 * This route is for creating a new item for a particular category. 
 * The category that the items belongs to is stored as a part of the item object in the database.
 */
app.post('/create/item/', (req, res) => {
  let newItemToSave = req.body;
  var newItem = new Item(newItemToSave);
  let p1 = newItem.save();
  p1.then( (doc) => { 
    res.end('SAVED SUCCESFULLY');
  });
  p1.catch( (err) => { 
    console.log(err);
    res.end('FAIL');
  });
});

/**
 * This route is for creating a new list category.
 */
app.post('/create/category/', (req, res) => {
  let CategoryToSave = req.body;
  var newCategory = new Category(CategoryToSave);
  let p1 = newCategory.save();
  p1.then( (doc) => { 
    res.end('SAVED SUCCESFULLY');
  });
  p1.catch( (err) => { 
    console.log(err);
    res.end('FAILED TO CREATE A CATEGORY');
  });
});

/**
 * This route is for fetching all of the categories stored in the database.
 */
app.get('/categories/', (req, res) => {
    let p1 = Category.find({}).exec();
    p1.then( (results) => { 
      res.end( JSON.stringify(results) );
    });
    p1.catch( (error) => {
      console.log(error);
      res.end('FAIL');
    });
});

app.post('/account/login', (req, res) => { 
  console.log(sessions);
  let u = req.body;
  let p1 = User.find({username: u.username, password: u.password}).exec();
  p1.then( (results) => { 
    if (results.length == 0) {
      res.end('Coult not find account');
    } else {
      let sid = addSession(u.username);  
      res.cookie("login", 
        {username: u.username, sessionID: sid}, 
        {maxAge: 60000 * 2 });
      res.end('SUCCESS');
    }
  });
});

app.get('/account/create/:user/:pass', (req, res) => { 
  
  let p1 = User.find({username: req.params.user}).exec();
  p1.then( (results) => { 
    if (results.length == 0) {
      let u = new User({ 
        username: req.params.user,
        password: req.params.pass,
        faveColor: 'red'
      });
      let p = u.save();
      p.then(() => { 
        res.end('USER CREATED');
      });
      p.catch( () => { 
        res.end('DATABASE SAVE ISSUE');
      });
    } else {
      res.end('USERNAME ALREADY TAKEN');
    }
  });

});
// Start up the server to listen on port 80
const port = 80;
app.listen(port, () => { console.log('server has started'); });








